---- Filter
-- @author Oskar Eisemuth
-- @copyright 2017-2021
-- @module commonapi.filter
local filter = {}

local _log = commonapi.utils.logPrep("commonapi2.filter")

local userfiltertracks = {}
local userfilterstreets = {}
local isprepared = false


local function splitUserFilterSettings(typ, userfilter)
	local userfilterString = commonapi.settings.get("eis_os_commonapi_1", "filter"..typ) or ""
	_log(typ..":", userfilterString)
	for e in string.gmatch(userfilterString, "[^;]+") do
		userfilter[#userfilter+1] = e
	end
end


local function prepareUserFilter()
	if (isprepared) then
		return
	end
	splitUserFilterSettings("tracks", userfiltertracks)
	splitUserFilterSettings("streets", userfilterstreets)
	isprepared = true
end

local function willBridgeCarry(data, typ)
	local i
	local carriers = data.carriers or {}
	for i = 1, #carriers do
		if (carriers[i] == typ) then
			return true
		end
	end
	return false
end

--- Filter hidden items
filter.visibleInUI = function(filename, data)
	if (data.hideInUI == true) then
		return false
	end
	local yearTo = data.yearTo or 0
	if (yearTo ~= 0 and yearTo <= 1850) then
		return false
	end
	return true
end

local function applyFilenameFilters(filters, filename)
	local f = string.gsub(filename, "%.lua$", "")
	for i, p in ipairs(filters) do
		if (string.find(f, p) ~= nil) then
			return false
		end
	end
	return true
end

filter._settingsTracks = function(filename, data)
	prepareUserFilter()
	if (#userfiltertracks == 0) then
		return true
	end
	return applyFilenameFilters(userfiltertracks, filename)
end

filter._settingStreet = function(filename, data)
	prepareUserFilter()
	if (#userfiltertracks == 0) then
		return true
	end
	return applyFilenameFilters(userfilterstreets, filename)
end

--- Keep only rail bridges
filter.bridgeCarryRail = function(filename, data)
	return willBridgeCarry(data, "RAIL")
end

--- Keep only road bridges
filter.bridgeCarryRoad = function(filename, data)
	return willBridgeCarry(data, "ROAD")
end
return filter