local ToolboxWindow = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.ui2.trackStreetToolbox")
local ui = require("commonapi2.ui2.tpfui")
ToolboxWindow = ui.Window:new()

ToolboxWindow.id =  "trackStreetToolbox"
ToolboxWindow.caption = "Track & Street Toolbox"
ToolboxWindow.uilist = {}

ToolboxWindow.height = 500
ToolboxWindow.width = 500

function ToolboxWindow.create()
	local o
	o = ToolboxWindow:new()
	o:createUIList()
	o:open()

	return o
end

function ToolboxWindow:refresh()
	self:createUIList()
end

function ToolboxWindow:createUIList()
	local uilist = {}
	local uitabs = {}

	uitabs[#uitabs+1] = {
		typ = "TabItem",
		id = "TrackTab",
		caption = self._("Track"),
		uilist = self:createUITrackTabEntity()
	}

	uitabs[#uitabs+1] = {
		typ = "TabItem",
		id = "StreetTab",
		caption = self._("Street"),
		uilist = self:createUIStreetTabEvents()
	}

	uilist[#uilist+1] = {
		typ = "TabBar",
		id = "Tabs",
		uilist = uitabs,
	}
	self.uilist = uilist
end

local function saveRepoData(reponame)
	if (commonapi._trackstreettoolboxdata == nil) then
		commonapi._trackstreettoolboxdata = {}
	else
		if (commonapi._trackstreettoolboxdata[reponame] ~= nil) then
			return
		end
	end
	local repo = api.res[reponame]
	local repoentries = repo.getAll()
	local savedata = {}
	for key, name in pairs(repoentries) do
		savedata[key] = repo.get(key)
	end
	if (commonapi._trackstreettoolboxdata == nil) then
		commonapi._trackstreettoolboxdata = {}
	end
	commonapi._trackstreettoolboxdata[reponame] = savedata
end


local function patchRepoData(reponame, restoredata, patchdata)
	if (commonapi._trackstreettoolboxdata == nil) then
		return
	end
	if (commonapi._trackstreettoolboxdata[reponame] == nil) then
		return
	end
	local reposic = commonapi._trackstreettoolboxdata[reponame]
	local reporw = api.res[reponame..'_CommonAPI2_RW']

	local repoentries = reporw.getAll()

	for idx, filename in pairs(repoentries) do
		local datarw = reporw.get(idx)
		local datasic = reposic[idx]

		for rdatakey, value in pairs(restoredata) do
			datarw[rdatakey] = datasic[rdatakey]
		end
		for datakey, value in pairs(patchdata) do
			_log("Set ", filename, " -> "..datakey.." = "..value)
			datarw[datakey] = value
		end
	end
end


ToolboxWindow.speedLimits = { 0, 10, 15, 20, 25, 30, 40, 50, 60, 100, 160, 200 };
ToolboxWindow.minCurveRadiusList = { 0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100 }
ToolboxWindow.slopes = { 0, 0.05, 0.075, 0.10, 0.15, 0.20, 0,25, 0.30, 0.35, 0.4, 0.45, 0.5 };
ToolboxWindow.embankmentSlopes = { 0, 0.75, 1, 1.25, 1.50, 1.75, 2.00, 2.25, 2.5, 3 };

--ToolboxWindow.slopeStrings  = { ToolboxWindow._("Default"), "0.75", "1", "1.25", "1.50", "1.75", "2.00", "2.25", "2.5", "3"};
--ToolboxWindow.slopes = { 0, 0.75, 1, 1.25, 1.50, 1.75, 2.00, 2.25, 2.5, 3 };



function ToolboxWindow:getComboStrings(values)
	local strings = {}
	for idx, value in ipairs(values) do
		strings[idx] = tostring(value)
	end
	if (values[1] == 0) then
		strings[1] = self._("Default")
	end
	return strings
end


function ToolboxWindow:getComboValue(name, values)
	if (self.params[name] == nil) then
		return nil
	end
	local comboIndex = self.params[name] or 0
	local value = values[comboIndex+1] or 0
	return value
end
function ToolboxWindow:changeSettings()
	saveRepoData("trackTypeRep")
	local restoreTracks = {
		speedLimit = true,

		maxSlopeBuild = true,
		maxSlope = true,
		maxSlopeShape = true,

		embankmentSlopeLow = true,
		embankmentSlopeHigh = true,
		minCurveRadius = true,
		minCurveRadiusBuild = true,
	}
	local patchTracks = {}

	local TrackSpeedLimit = self:getComboValue("TrackSpeedLimit", self.speedLimits)
	local TrackMaxSlope = self:getComboValue("TrackMaxSlope", self.slopes)
	local TrackEmbankmentSlope = self:getComboValue("TrackEmbankmentSlope", self.embankmentSlopes)
	local TrackMinCurveRadius =  self:getComboValue("TrackMinCurveRadius", self.minCurveRadiusList)

	if (TrackSpeedLimit and TrackSpeedLimit > 0) then
		_log("changeSettings() speedLimit = ", TrackSpeedLimit)
		patchTracks["speedLimit"] = TrackSpeedLimit / 3.6
	end

	if (TrackEmbankmentSlope and TrackEmbankmentSlope > 0) then
		patchTracks["embankmentSlopeLow"] = TrackEmbankmentSlope
		patchTracks["embankmentSlopeHigh"] = TrackEmbankmentSlope
	end

	if (TrackMaxSlope and TrackMaxSlope > 0) then
		patchTracks.maxSlopeBuild = TrackMaxSlope -- 0.075
		patchTracks.maxSlope = patchTracks.maxSlopeBuild * 1.6
		patchTracks.maxSlopeShape = patchTracks.maxSlope * 1.25
	end

	if (TrackMinCurveRadius and TrackMinCurveRadius > 0) then
		patchTracks.minCurveRadius = TrackMinCurveRadius
		patchTracks.minCurveRadiusBuild = TrackMinCurveRadius * 1.4
	end

	patchRepoData("trackTypeRep", restoreTracks, patchTracks)
end

function ToolboxWindow:createUITrackTabEntity()
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = self._("Speedlimit")
	}

	uilist[#uilist+1] = {
		typ = "ComboBox",
		id = "TrackSpeedLimit",
		values = self:getComboStrings(self.speedLimits),
		onclick = function(item, win)
			win:changeSettings()
		end
	}


	uilist[#uilist+1] = {
		typ = "Label",
		caption = self._("Embankment")
	}

	uilist[#uilist+1] = {
		typ = "ComboBox",
		id = "TrackEmbankmentSlope",
		values = self:getComboStrings(self.embankmentSlopes),
		onclick = function(item, win)
			win:changeSettings()
		end
	}

	uilist[#uilist+1] = {
		typ = "Label",
		caption = self._("Slope")
	}

	uilist[#uilist+1] = {
		typ = "ComboBox",
		id = "TrackMaxSlope",
		values = self:getComboStrings(self.slopes),
		onclick = function(item, win)
			win:changeSettings()
		end
	}


	uilist[#uilist+1] = {
		typ = "Label",
		caption = self._("Curve radius")
	}

	uilist[#uilist+1] = {
		typ = "ComboBox",
		id = "TrackMinCurveRadius",
		values = self:getComboStrings(self.minCurveRadiusList),
		onclick = function(item, win)
			win:changeSettings()
		end
	}

	return uilist
end


function ToolboxWindow:createUIStreetTabEvents()
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Separator"
	}

	return uilist
end

return ToolboxWindow